import torch
import numpy as np
import librosa


class SpeakerInformation(object):

    def __init__(self, threshold):
        self.model = torch.hub.load('RF5/simple-speaker-embedding', 'convgru_embedder')
        self.threshold = threshold
        self._current_embedding = None
        self._stored_embeddings_unique = []
        # we assume there is a default first speaker
        self._stored_embeddings_unique = np.zeros((1, 256))
        self._label_drawer = 1
        self.label_sequence = []

    def addaudio(self, filepath):

        wav, _ = librosa.load(filepath, sr=16000)
        wav = torch.from_numpy(wav).float()
        embedding = self.model(wav[None])
        self._current_embedding = np.reshape(embedding[0].cpu().detach().numpy(), (1, 256))

    def existedornot(self):
        # return number of audios
        distance = (np.sum((self._current_embedding - self._stored_embeddings_unique) ** 2, axis=1)) ** (1 / 2)
        if (np.min(distance) > self.threshold):
            print("we have a new speaker!")
            self.label_sequence.append(self._label_drawer)
            self._label_drawer = self._label_drawer + 1
            self._stored_embeddings_unique = np.vstack([self._stored_embeddings_unique, self._current_embedding])
        else:
            print("we have this speaker presented already!")
            speaker_label = np.argmin(distance)
            self.label_sequence.append(speaker_label)
        return self.label_sequence


def example():
    voice = SpeakerInformation(0.65)
    voice.addaudio('../audio/a1.wav')
    label = voice.existedornot()
    print(label)
    voice.addaudio('../audio/a2.wav')
    label = voice.existedornot()
    print(label)
    voice.addaudio('../audio/c1.wav')
    label = voice.existedornot()
    print(label)
    voice.addaudio('../audio/b2.wav')
    label = voice.existedornot()
    print(label)
    voice.addaudio('../audio/c1.wav')
    label = voice.existedornot()
    print(label)
    # voice.addaudio('wav_example/10288/00005.wav')
    # label = voice.existedornot()
    # print(label)


if __name__ == "__main__":
    example()
