import os
import wave
import deepspeech
import numpy as np

from django.shortcuts import render, redirect
from django.conf import settings

from .models import AudioFile
from .forms import AudioFileForm

from .experiment import SpeakerInformation

def home(request):
    audio_files = AudioFile.objects.all()
    context = {'audio_files': audio_files}
    return render(request, 'base/home.html', context)


def upload_file(request):
    if request.method == 'POST':
        form = AudioFileForm(request.POST, request.FILES)
        if form.is_valid():
            object = form.save()
            model_file_path = 'deepspeech-0.9.3-models.pbmm'
            ds = deepspeech.Model(model_file_path)
            audio = object.get_path()
            file = wave.open(audio, 'rb')
            audio = np.frombuffer(file.readframes(file.getnframes()), np.int16)
            object.set_trans(ds.stt(audio))
            object.save()

            # label user
            audio_files = AudioFile.objects.all()
            voice = SpeakerInformation(0.65) # change parameter here to alter model setting
            for i in range(len(audio_files)-1,-1,-1): # reverse reading from database
                audio_file = audio_files[i]
                voice.addaudio(audio_file.get_path())
                label = voice.existedornot()
                audio_file.set_userNum(label)
                audio_file.save()

            return redirect('files')
    else:
        form = AudioFileForm()

    context = {'form': form}
    return render(request, 'base/upload_file.html', context)


def delete_file(request, audio_file_id):
    audio_file_to_delete = AudioFile.objects.get(id=audio_file_id)

    # delete file
    os.remove(os.path.join(settings.MEDIA_ROOT, audio_file_to_delete.file.name))

    audio_file_to_delete.delete()
    return redirect('files')


def check_files(request):
    audio_files = AudioFile.objects.all()
    context = {'audio_files': audio_files}
    return render(request, 'base/files.html', context)


def record_audio(request):
    return render(request, 'base/record.html')
